package org.hso.ecommerce.uimodel;

import com.fasterxml.jackson.annotation.JsonCreator;
import org.hso.ecommerce.api.RestServiceForDelivery;
import org.hso.ecommerce.entities.shop.CustomerOrder;
import org.hso.ecommerce.repos.shop.CustomerOrderRepository;

import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.UUID;


public class DeliveryData
{

    private final String status;

    private final String estimatedArrival;

    private boolean isDelivered;

    @JsonCreator(mode = JsonCreator.Mode.PROPERTIES)
    public DeliveryData(String status, String estimatedArrival) {
        this.status = status;
        this.estimatedArrival = estimatedArrival;
        isDelivered = status.equals("Lieferung erfolgreich");
    }

    public static  DeliveryData getDeliveryDataFromCustomerOrder(CustomerOrder customerOrder, CustomerOrderRepository customerOrderRepository, RestServiceForDelivery restServiceForDelivery)
    {
        if(customerOrder.trackingId == null)
            return new DeliveryData("Bestellung wurde elektronisch angekündigt","");

        if(customerOrder.deliveredAt == null)
        {
            DeliveryData deliveryData = restServiceForDelivery.getDeliveryData(UUID.fromString(customerOrder.trackingId));

            if(deliveryData.isDelivered())
            {
                Calendar calendar = Calendar.getInstance();

                SimpleDateFormat simpleDateFormat = new SimpleDateFormat("dd-MM-yyyy");

                try {
                    calendar.setTime(simpleDateFormat.parse(deliveryData.getEstimatedArrival()));
                } catch (ParseException e) {
                    e.printStackTrace();
                }

                customerOrder.deliveredAt = new Timestamp(calendar.getTimeInMillis());
                customerOrderRepository.save(customerOrder);
            }

            return deliveryData;
        }

        SimpleDateFormat formatter = new SimpleDateFormat("dd-MM-yyyy");
        return new DeliveryData("Lieferung erfolgreich",formatter.format(customerOrder.deliveredAt));
    }

    public boolean isDelivered() {
        return isDelivered;
    }

    public String getStatus() {
        return status;
    }

    public String getEstimatedArrival() {
        return estimatedArrival;
    }
}

